/*
    Copyright (c) 2009 by Joseph Devietti (devietti@cs.washington.edu).

    This file is part of my DMP ASPLOS 2009 Presentation (DAP).

    DAP is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DAP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DAP.  If not, see <http://www.gnu.org/licenses/>.
*/ 

﻿package {
    /* Import the required classes. Note that since this is an external .AS file and 
       not an .FLA file, most classes will not be automatically imported for you. */
	import fl.controls.listClasses.*;
	import flash.display.DisplayObject;
	import flash.display.Sprite;
	import flash.geom.Rectangle;
    import flash.text.TextFormat;
    import flash.text.TextFormatAlign;

    /**
     * The main class definition. Make sure the class is marked "public" and in the case
     * of our custom cell renderer, extends the CellRenderer class and implements the
     * ICellRenderer interface.
     */
    public class CellStrikethroughRenderer extends CellRenderer implements ICellRenderer {
        // Create a new private variable to hold the custom text format.
        private var tf:TextFormat;

        /**
         * This method defines the TextFormat object and sets the align
         * property to "right".
         */
        public function CellStrikethroughRenderer() {
            tf = new TextFormat();
        }

        /**
         * Override the inherited drawLayout() method from the CellRenderer class.
         * This method draws strikethrough text for all but the last token of the textField. 
         */
        override protected function drawLayout():void {
			var text:String = textField.text;
			var tokens:Array = text.split( " " );
			var token_start:int = -1;
			var strikethru:Sprite = new Sprite();
			strikethru.name = "jld_strikethru";

			// remove any existing strikethroughs
			var old:DisplayObject = this.getChildByName( "jld_strikethru" );
			if ( null != old ) {
				this.removeChild( old );
			}
			
			// %height at which the strikethrough line is drawn
			const ST_HEIGHT:Number = 0.4;
			
			for ( var i:int = 0; i < tokens.length - 1/*don't touch last token*/; i++ ) {
				token_start = text.indexOf( tokens[i], token_start+1 );
				
				var token_bb:Rectangle = null;
				for ( var j:int = 0; j < tokens[i].length; j++ ) {
					var char_bb:Rectangle = textField.getCharBoundaries( token_start + j );
					token_bb = ( null == token_bb ) ? char_bb : token_bb.union(char_bb) ;
				}
				// token_bb is in textField's coord space
				//trace( "token bb: " + token_bb.toString() );
				strikethru.graphics.lineStyle( 2, 0x0 );
				strikethru.graphics.moveTo( token_bb.left, token_bb.bottom - (ST_HEIGHT*token_bb.height) );
				strikethru.graphics.lineTo( token_bb.right, token_bb.bottom - (ST_HEIGHT*token_bb.height) );
			}
			this.addChild( strikethru );
			super.drawLayout();
        }
    }
}
